/*---------------------------------------------------------------------------*/

/*!  \file  frisca_cg.c
     \brief  a simple rheology simulator based on the CG slip-spring model.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2009-2011 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include <stdio.h>
#include "frisca_cg.h"

/*---------------------------------------------------------------------------*/
int main(int argc,char **argv)
{
        /*  simulaton parameters
              iterations: number of iterations per one simulator execution
              p: total number of simulation executions
                 (total number of time steps = iterations * p)
              n: number of CG segments per chain
              m: number of chains
              n0: average CG segment number between CG slip-springs
              n_s: CG slip-spring strength parameter
                   (effective strength is calculated by n_s)
              zeta_s: effective friction coefficent for a CG slip-spring
              dt: time step size
              seed: random seed
              kappa: velocity gradient tensor  */
    int iterations = 100;
    int p = 1000;
    int n = 20;
    int m = 512;
    double n0 = 1.0;
    double n_s = 0.5;
    double zeta_s = 0.1;
    double dt = 0.01;
    unsigned long seed = 123402;
    tensor3 kappa = {{0, 0.1, 0},
                     {0, 0, 0},
                     {0, 0, 0}};

    int i;
    simulator_state *state;
    symmetric_tensor3 sigma;

    state = allocate_simulator_state(m,n);

    initialize_simulator_state(state,seed,n0,n_s,zeta_s);


    for(i = 0;i < p;i++)
    {
        get_stress_tensor(state,sigma);
        printf("%f %f %f %f %f %f\n",
               sigma[0],sigma[1],sigma[2],sigma[3],sigma[4],sigma[5]);

        perform_simulation(state,iterations,dt,kappa);
    }

    store_simulator_state_to_file("frisca_cg_state.h5",state);

    free_simulator_state(state);

    return 0;
}

/*---------------------------------------------------------------------------*/
