/*---------------------------------------------------------------------------*/

/*!  \file  chain.h
     \brief  definitions and subroutines for slip-springed chains.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2011 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_CHAIN_H
#define INCLUDE_CHAIN_H

#include "rng.h"

/*---------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C"
{
#endif
    typedef struct slip_springed_chain
    {
        int m,n;
        float *r_host,*r_device;
        float *a_host,*a_device;
        int *phi_host,*phi_device;
    } slip_springed_chain;
#ifdef __cplusplus
}
#endif

/*---------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C"
{
#endif
    slip_springed_chain *allocate_slip_springed_chain(int m,int n);
    void free_slip_springed_chain(slip_springed_chain *chain);
    void initialize_slip_springed_chain(slip_springed_chain *chain,
                                        rng_state *rng,float n0,float n_s);
    void copy_slip_springed_chain_host_to_device(slip_springed_chain *chain);
    void copy_slip_springed_chain_device_to_host(slip_springed_chain *chain);
#ifdef __cplusplus
}
#endif

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
