/*---------------------------------------------------------------------------*/

/*!  \file  chain.h
     \brief  definitions and subroutines for slip-springed chains.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2010 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_CHAIN_H
#define INCLUDE_CHAIN_H

#include "vector.h"
#include "rng.h"

/*---------------------------------------------------------------------------*/
typedef struct slip_springed_chain
{
    int n,m;
    double n0,n_s_bar,zeta_s_inv;
    vector3 *r;
    vector3 *a;
    int *phi;
    vector3 *w,*f;
} slip_springed_chain;

/*---------------------------------------------------------------------------*/
slip_springed_chain *allocate_slip_springed_chain(int m,int n);
void free_slip_springed_chain(slip_springed_chain *chain);
void initialize_slip_springed_chain(slip_springed_chain *chain,rng_state *rng,
                                    double n0,double n_s,double zeta_s);
void calculate_average_stress_tensor(slip_springed_chain *chain,
                                     symmetric_tensor3 sigma);
void calculate_average_virtual_stress_tensor(slip_springed_chain *chain,
                                             symmetric_tensor3 sigma);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
