/*---------------------------------------------------------------------------*/

/*!  \file  frisca_gpu.h
     \brief  utility functions to handle the simulator  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2009-2011 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_FRISCA_GPU_H
#define INCLUDE_FRISCA_GPU_H

/*---------------------------------------------------------------------------*/
#include "chains.h"
#include "springs.h"
#include "rng.h"
#include "rng_gpu.h"

/*---------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C"
{
#endif
    typedef struct simulator_state
    {
        int m,n,z_max;
        float n0,n_s;
        float dt,zeta_s_inv;
        float kappa[3][3];
        int iterations;
        float sigma[6];
        polymer_chains *chains;
        slip_springs *springs;
        rng_state *rng;
        rng_state_gpu *rng_gpu;
        float *sigma_subtotal_device[2];
    } simulator_state;
#ifdef __cplusplus
}
#endif

/*---------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C"
{
#endif
    simulator_state *allocate_simulator_state(int m,int n,int z_max);
    void free_simulator_state(simulator_state *state);
    void initialize_simulator_state(simulator_state *state,unsigned long seed,
                                    float n_s,float n0);
    void set_simulation_parameters(simulator_state *state,float n_s,
                                   float n0,float zeta_s,float dt,
                                   float kappa[3][3],int iterations);
    void copy_simulator_state_host_to_device(simulator_state *state);
    void copy_simulator_state_parameters_host_to_device(simulator_state *state);
    void copy_simulator_state_device_to_host(simulator_state *state);
    void initialize_device(int argc,char **argv);
    void initialize_device_with_load_balancing(int id);
    void finalize_device(void);
    void perform_simulation(simulator_state *state);
    void calculate_stress_tensor(simulator_state *state);
    void set_centers_of_mass_to_origin(simulator_state *state);
    void store_simulator_state_to_file(char *h5filename,
                                       simulator_state *state);
    simulator_state *load_simulator_state_from_file(char *h5filename);
#ifdef __cplusplus
}
#endif

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
