/*---------------------------------------------------------------------------*/

/*!  \file  frisca.h
     \brief  a single chain slip-spring model simulator.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2009-2011 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_FRISCA_H
#define INCLUDE_FRISCAE_H

/*---------------------------------------------------------------------------*/
#include "chain.h"
#include "springs.h"
#include "rng.h"

/*---------------------------------------------------------------------------*/
typedef struct simulator_state
{
    int m;
    polymer_chain **chain;
    slip_springs **springs;
    rng_state *rng;
} simulator_state;

/*---------------------------------------------------------------------------*/
simulator_state *allocate_simulator_state(int m,int n,int z_max);
void free_simulator_state(simulator_state *state);
void initialize_simulator_state(simulator_state *state,unsigned long seed,
                                double n_s,double n0,double zeta_s);
void perform_simulation(simulator_state *state,int iterations,double dt,
                        tensor3 kappa);
void set_centers_of_mass_to_origin(simulator_state *state);
void get_stress_tensor(simulator_state *state,symmetric_tensor3 sigma);
void store_simulator_state_to_file(char *h5filename,simulator_state *state);
simulator_state *load_simulator_state_from_file(char *h5filename);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
