/*---------------------------------------------------------------------------*/

/*!  \file  jump_kernel.c
     \brief  kernel for jump.c.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2009-2011 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#define KERNEL2(x,y) x ## _ ## y
#define KERNEL(x,y) KERNEL2(x,y)

/*---------------------------------------------------------------------------*/
static void KERNEL(reconstruct_slip_springs,SUFFIX)(
    polymer_chain *chain,slip_springs *springs,rng_state *rng,double dt)
{
    int i,j;
    double u;
    double w_destruct,w_construct,w;
    int z_new;
#ifdef ENABLE_CR
    double tau_s_inv;
    double w_cr_destruct,w_cr_construct;
#endif
#ifdef BRANCHED_CHAIN
    double *zeta_b_inv;
#endif

#ifdef ENABLE_CR
    tau_s_inv = springs->tau_s_inv;
    w_cr_destruct = dt * tau_s_inv;
    w_cr_construct = dt * tau_s_inv * chain->n / springs->n0;
#endif
#ifdef BRANCHED_CHAIN
    zeta_b_inv = chain->zeta_b_inv;
#endif
    w_destruct = dt * springs->zeta_s_inv;
    w_construct = w_destruct / springs->n0;
    z_new = springs->z;

        /*  destruct slip-springs  */
    for(i = 0;i < springs->z;i++)
    {
        j = springs->s[i];

#ifdef ENABLE_CR
            /*  constructions at chain ends and by the constraint release  */
        w = w_cr_destruct;

        u = urand(rng);
#ifdef BRANCHED_CHAIN
        if(j == 0)
        {
            w += w_destruct * zeta_b_inv[0];
        }
        else if(j == (chain->n - 1))
        {
            w += w_destruct * zeta_b_inv[1];
        }
#else
        if((j == 0) || (j == (chain->n - 1)))
        {
            w += w_destruct;
        }
#endif

#else
            /*  constructions only at chain ends (no constraint release)  */
        if((j == 0) || (j == (chain->n - 1)))
        {
#ifdef BRANCHED_CHAIN
            if(j == 0)
            {
                w = w_destruct * zeta_b_inv[0];
            }
            else
            {
                w = w_destruct * zeta_b_inv[1];
            }
#else
            w = w_destruct;
#endif
            u = urand(rng);
#endif

            if(u < w)
            {
                springs->s[i] = NO_BEAD_INDEX;
                springs->a[i][0] = 0.0;
                springs->a[i][1] = 0.0;
                springs->a[i][2] = 0.0;
                
                z_new--;
            }
#ifndef ENABLE_CR
        }
#endif
    }

        /*  sort the slip-spring array to make it compact  */
    if(z_new != springs->z)
    {
        for(i = 0;i < z_new;i++)
        {
            if(springs->s[i] == NO_BEAD_INDEX)
            {
                for(j = springs->z - 1;j > i;j--)
                {
                    if(springs->s[j] != NO_BEAD_INDEX)
                    {
                        swap_two_springs(springs,i,j);
                        break;
                    }
                }
            }
        }

        springs->z = z_new;
    }


        /*  construct slip-springs at chain ends  */
#ifdef BRANCHED_CHAIN
    w = w_construct * zeta_b_inv[0];
#else
    w = w_construct;
#endif
    u = urand(rng);
    if((u < w) && (springs->z < springs->z_max))
    {
        i = springs->z;

        springs->s[i] = 0;
        springs->a[i][0] = chain->r[0][0]
            + nrand(rng) * sqrt(springs->n_s / 3.0);
        springs->a[i][1] = chain->r[0][1]
            + nrand(rng) * sqrt(springs->n_s / 3.0);
        springs->a[i][2] = chain->r[0][2]
            + nrand(rng) * sqrt(springs->n_s / 3.0);

        springs->z++;
    }

#ifdef BRANCHED_CHAIN
    w = w_construct * zeta_b_inv[1];
#endif
    u = urand(rng);
    if((u < w) && (springs->z < springs->z_max))
    {
        i = springs->z;

        springs->s[i] = chain->n - 1;
        springs->a[i][0] = chain->r[chain->n - 1][0]
            + nrand(rng) * sqrt(springs->n_s / 3.0);
        springs->a[i][1] = chain->r[chain->n - 1][1]
            + nrand(rng) * sqrt(springs->n_s / 3.0);
        springs->a[i][2] = chain->r[chain->n - 1][2]
            + nrand(rng) * sqrt(springs->n_s / 3.0);

        springs->z++;
    }

        /*  construction by the constraint release  */
#ifdef ENABLE_CR
    w = w_cr_construct;
    u = urand(rng);
    if((u < w) && (springs->z < springs->z_max))
    {
        i = springs->z;
        j = (int)(chain->n * urand(rng));

        springs->s[i] = j;
        springs->a[i][0] = chain->r[j][0]
            + nrand(rng) * sqrt(springs->n_s / 3.0);
        springs->a[i][1] = chain->r[j][1]
            + nrand(rng) * sqrt(springs->n_s / 3.0);
        springs->a[i][2] = chain->r[j][2]
            + nrand(rng) * sqrt(springs->n_s / 3.0);

        springs->z++;
    }
#endif
}

/*---------------------------------------------------------------------------*/
#undef KERNEL
#undef KERNEL2

/*---------------------------------------------------------------------------*/
