/*---------------------------------------------------------------------------*/

/*!  \file  rng.c
     \brief  common random number generator  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2009 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include <stdlib.h>
#include <math.h>
#include "rng.h"
#include "mt.h"

/*---------------------------------------------------------------------------*/
rng_state *allocate_rng_state(void)
{
    rng_state *rng;

    rng = malloc(sizeof(rng_state));
    rng->mt = allocate_mt_state();
    rng->nrand_has_saved_number = 0;

    return rng;
}

/*---------------------------------------------------------------------------*/
void free_rng_state(rng_state *rng)
{
    free_mt_state(rng->mt);
    free(rng);
}

/*---------------------------------------------------------------------------*/
void initialize_rng_state(rng_state *rng,unsigned long seed)
{
    mtinit(rng->mt,seed);
}

/*---------------------------------------------------------------------------*/
double urand(rng_state *rng)
{
    return mtrand_real2(rng->mt);
}

/*---------------------------------------------------------------------------*/
double nrand(rng_state *rng)
{
    double r,theta;

    if(rng->nrand_has_saved_number)
    {
        rng->nrand_has_saved_number = 0;

        return rng->nrand_w;
    }
    else
    {
        r = sqrt(-2.0 * log(1.0 - mtrand_real2(rng->mt)));
        theta = 2.0 * M_PI * mtrand_real2(rng->mt);

        rng->nrand_has_saved_number = 1;
        rng->nrand_w = r * sin(theta);

        return (r * cos(theta));
    }
}

/*---------------------------------------------------------------------------*/
int prand(rng_state *rng,double mu)
{
    double x;
    int n;

    n = 0;
    x = -log(1.0 - mtrand_real2(rng->mt));

    while(x < mu)
    {
        x += -log(1.0 - mtrand_real2(rng->mt));
        n++;
    }

    return n;
}

/*---------------------------------------------------------------------------*/
