/*---------------------------------------------------------------------------*/

/*!  \file  springs.h
     \brief  definitions and subroutines for slip-springs.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2009-2011 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_SPRINGS_H
#define INCLUDE_SPRINGS_H

/*---------------------------------------------------------------------------*/
#include "vector.h"
#include "chain.h"
#include "rng.h"

/*---------------------------------------------------------------------------*/
#define NO_BEAD_INDEX (-1)

/*---------------------------------------------------------------------------*/
typedef struct slip_springs
{
    int z_max;
    int z;
    double n0;
    double n_s;
    double zeta_s_inv;
    double tau_s_inv;
    vector3 *a;
    vector3 *a_star;
    int *s;
} slip_springs;

/*---------------------------------------------------------------------------*/
slip_springs *allocate_slip_springs(int z_max);
void free_slip_springs(slip_springs *springs);
void initialize_slip_springs(slip_springs *springs,polymer_chain *chain,
                             rng_state *rng,double n_s,double n0,
                             double zeta_s,double tau_s);
int calculate_estimate_for_z_max(int n,double n0);
void calculate_virtual_stress_tensor(polymer_chain *chain,
                                     slip_springs *springs,
                                     symmetric_tensor3 sigma);
void calculate_average_virtual_stress_tensor(polymer_chain **chain,
                                             slip_springs **springs,
                                             int m,
                                             symmetric_tensor3 sigma);
double calculate_average_number_of_slip_springs(slip_springs **springs,int m);
void sort_slip_springs_by_bead_index(slip_springs *springs);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
