/*---------------------------------------------------------------------------*/

/*!  \file  strain.c
     \brief  apply strains to the system and deform it.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2009 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include <math.h>
#include "chain.h"
#include "springs.h"

/*---------------------------------------------------------------------------*/
void apply_strain(polymer_chain *chain,slip_springs *springs,tensor3 e)
{
    int i,j;
    vector3 r_new,a_new;

    for(i = 0;i < chain->n;i++)
    {
        for(j = 0;j < 3;j++)
        {
            r_new[j] = e[j][0] * chain->r[i][0]
                + e[j][1] * chain->r[i][1]
                + e[j][2] * chain->r[i][2];
        }
        for(j = 0;j < 3;j++)
        {
            chain->r[i][j] = r_new[j];
        }
    }

    for(i = 0;i < springs->z;i++)
    {
        for(j = 0;j < 3;j++)
        {
            a_new[j] = e[j][0] * springs->a[i][0]
                + e[j][1] * springs->a[i][1]
                + e[j][2] * springs->a[i][2];
        }
        for(j = 0;j < 3;j++)
        {
            springs->a[i][j] = a_new[j];
        }
    }
}

/*---------------------------------------------------------------------------*/
void apply_simple_shear_strain(polymer_chain *chain,slip_springs *springs,
                               double gamma_xy)
{
    int i;

    for(i = 0;i < chain->n;i++)
    {
        chain->r[i][0] += gamma_xy * chain->r[i][1];
    }

    for(i = 0;i < springs->z;i++)
    {
        springs->a[i][0] += gamma_xy * springs->a[i][1];
    }
}

/*---------------------------------------------------------------------------*/
void apply_simple_uniaxial_strain(polymer_chain *chain,slip_springs *springs,
                                  double lambda_z)
{
    int i;

    for(i = 0;i < chain->n;i++)
    {
        chain->r[i][0] /= sqrt(lambda_z);
        chain->r[i][1] /= sqrt(lambda_z);
        chain->r[i][2] *= lambda_z;
    }

    for(i = 0;i < springs->z;i++)
    {
        springs->a[i][0] /= sqrt(lambda_z);
        springs->a[i][1] /= sqrt(lambda_z);
        springs->a[i][2] *= sqrt(lambda_z);
    }
}

/*---------------------------------------------------------------------------*/
